@@ -1,16 +1,18 @@
 """Defines electromagnetic boundary conditions"""
+from __future__ import annotations
+
 from abc import ABC
 from typing import Union, Tuple, List
 
 import pydantic as pd
 import numpy as np
 
-from .base import Tidy3dBaseModel
+from .base import Tidy3dBaseModel, cached_property
 from .types import Complex, Axis, Literal
 from .source import GaussianBeam, ModeSource, PlaneWave
 from .medium import Medium
 
-from ..log import log, SetupError, DataError
+from ..log import log, SetupError, DataError, ValidationError
 from ..constants import EPSILON_0, MU_0, PML_SIGMA
 
 
@@ -57,7 +59,7 @@ class BlochBoundary(BoundaryEdge):
         "along the dimension in which the boundary is specified.",
     )
 
-    @property
+    @cached_property
     def bloch_phase(self) -> Complex:
         """Returns the forward phase factor associated with `bloch_vec`."""
         return np.exp(1j * 2.0 * np.pi * self.bloch_vec)
@@ -65,8 +67,11 @@ class BlochBoundary(BoundaryEdge):
     @classmethod
     def from_source(
         cls, source: BlochSourceType, domain_size: float, axis: Axis, medium: Medium = None
-    ) -> "BlochBoundary":
+    ) -> BlochBoundary:
         """Set the Bloch vector component based on a given angled source and its center frequency.
+           Note that if a broadband angled source is used, only the frequency components near the
+           center frequency will exhibit angled incidence at the expect angle. In this case, a
+           narrowband source is recommended.
 
         Parameters
         ----------
@@ -111,7 +116,7 @@ class BlochBoundary(BoundaryEdge):
 
         freq0 = source.source_time.freq0
         eps_complex = medium.eps_model(freq0)
-        kmag = freq0 * np.sqrt(eps_complex * EPSILON_0 * MU_0)
+        kmag = np.real(freq0 * np.sqrt(eps_complex * EPSILON_0 * MU_0))
 
         angle_theta = source.angle_theta
         angle_phi = source.angle_phi
@@ -134,6 +139,13 @@ class BlochBoundary(BoundaryEdge):
         bloch_vec = domain_size * k_global[axis]
         return cls(bloch_vec=bloch_vec)
 
+    @pd.validator("bloch_vec", always=True)
+    def _zero_imaginary_part(cls, val):
+        """Make sure the imaginary part of the bloch vector is 0."""
+        if np.imag(val) != 0:
+            raise ValidationError("Bloch vector must be real.")
+        return val
+
 
 # """ Absorber parameters """
 
@@ -345,7 +357,7 @@ class Boundary(Tidy3dBaseModel):
         description="Boundary condition on the minus side along a dimension.",
     )
 
-    @pd.root_validator
+    @pd.root_validator(skip_on_failure=True)
     def bloch_on_both_sides(cls, values):
         """Error if a Bloch boundary is applied on only one side."""
         plus = values.get("plus")
@@ -357,7 +369,7 @@ class Boundary(Tidy3dBaseModel):
             )
         return values
 
-    @pd.root_validator
+    @pd.root_validator(skip_on_failure=True)
     def periodic_with_pml(cls, values):
         """Error if PBC is specified with a PML."""
         plus = values.get("plus")
@@ -370,7 +382,7 @@ class Boundary(Tidy3dBaseModel):
             raise SetupError("Cannot have both PML and PBC along the same dimension.")
         return values
 
-    @pd.root_validator
+    @pd.root_validator(skip_on_failure=True)
     def periodic_with_pec_pmc(cls, values):
         """If a PBC is specified along with PEC or PMC on the other side, manually set the PBC
         to PEC or PMC so that no special treatment of halos is required."""
@@ -619,7 +631,7 @@ class BoundarySpec(Tidy3dBaseModel):
 
         Example
         -------
-        boundaries = BoundarySpec.pml(y=True)
+        >>> boundaries = BoundarySpec.pml(y=True)
         """
         return cls(
             x=Boundary.pml() if x else Boundary.periodic(),
@@ -642,7 +654,7 @@ class BoundarySpec(Tidy3dBaseModel):
 
         Example
         -------
-        boundaries = BoundarySpec.pec(x=True, z=True)
+        >>> boundaries = BoundarySpec.pec(x=True, z=True)
         """
         return cls(
             x=Boundary.pec() if x else Boundary.periodic(),
@@ -665,7 +677,7 @@ class BoundarySpec(Tidy3dBaseModel):
 
         Example
         -------
-        boundaries = BoundarySpec.pmc(x=True, z=True)
+        >>> boundaries = BoundarySpec.pmc(x=True, z=True)
         """
         return cls(
             x=Boundary.pmc() if x else Boundary.periodic(),
@@ -684,7 +696,7 @@ class BoundarySpec(Tidy3dBaseModel):
 
         Example
         -------
-        boundaries = BoundarySpec.all_sides(boundary=PML())
+        >>> boundaries = BoundarySpec.all_sides(boundary=PML())
         """
         return cls(
             x=Boundary(minus=boundary, plus=boundary),
@@ -692,7 +704,7 @@ class BoundarySpec(Tidy3dBaseModel):
             z=Boundary(minus=boundary, plus=boundary),
         )
 
-    @property
+    @cached_property
     def to_list(self) -> List[Tuple[BoundaryEdgeType, BoundaryEdgeType]]:
         """Returns edge-wise boundary conditions along each dimension for internal use."""
         return [
