@@ -10,18 +10,19 @@ import matplotlib.pylab as plt
 import matplotlib as mpl
 from mpl_toolkits.axes_grid1 import make_axes_locatable
 
-from .base import cache
+from .base import cached_property
 from .validators import assert_unique_names, assert_objects_in_sim_bounds
 from .validators import validate_mode_objects_symmetry
 from .geometry import Box
-from .types import Ax, Shapely, FreqBound, GridSize, Axis
+from .types import Ax, Shapely, FreqBound, GridSize, Axis, annotate_type
 from .grid import Coords1D, Grid, Coords, GridSpec, UniformGrid
 from .medium import Medium, MediumType, AbstractMedium, PECMedium
 from .boundary import BoundarySpec, Symmetry, BlochBoundary, PECBoundary, PMCBoundary
 from .boundary import PML, StablePML, Absorber
 from .structure import Structure
 from .source import SourceType, PlaneWave, GaussianBeam, AstigmaticGaussianBeam
-from .monitor import MonitorType, Monitor, FreqMonitor, AbstractFieldMonitor
+from .monitor import MonitorType, Monitor, FreqMonitor
+from .monitor import AbstractFieldMonitor
 from .viz import add_ax_if_none, equal_aspect
 
 from .viz import MEDIUM_CMAP, PlotParams, plot_params_symmetry
@@ -31,7 +32,6 @@ from .viz import plot_params_pec, plot_params_pmc, plot_params_bloch
 from ..version import __version__
 from ..constants import C_0, MICROMETER, SECOND, inf
 from ..log import log, Tidy3dKeyError, SetupError, ValidationError
-from ..static import make_static
 from ..updater import Updater
 
 # minimum number of grid points allowed per central wavelength in a medium
@@ -132,18 +132,18 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         "determine the symmetry value.",
     )
 
-    structures: List[Structure] = pydantic.Field(
-        [],
+    structures: Tuple[Structure, ...] = pydantic.Field(
+        (),
         title="Structures",
-        description="List of structures present in simulation. "
+        description="Tuple of structures present in simulation. "
         "Note: Structures defined later in this list override the "
         "simulation material properties in regions of spatial overlap.",
     )
 
-    sources: List[SourceType] = pydantic.Field(
-        [],
+    sources: Tuple[annotate_type(SourceType), ...] = pydantic.Field(
+        (),
         title="Sources",
-        description="List of electric current sources injecting fields into the simulation.",
+        description="Tuple of electric current sources injecting fields into the simulation.",
     )
 
     boundary_spec: BoundarySpec = pydantic.Field(
@@ -152,10 +152,10 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         description="Specification of boundary conditions along each dimension.",
     )
 
-    monitors: List[MonitorType] = pydantic.Field(
-        [],
+    monitors: Tuple[annotate_type(MonitorType), ...] = pydantic.Field(
+        (),
         title="Monitors",
-        description="List of monitors in the simulation. "
+        description="Tuple of monitors in the simulation. "
         "Note: monitor names are used to access data after simulation is run.",
     )
 
@@ -181,6 +181,14 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         "based on structure definition, resulting in much higher accuracy for a given grid size.",
     )
 
+    normalize_index: Union[pydantic.NonNegativeInt, None] = pydantic.Field(
+        0,
+        title="Normalization index",
+        description="Index of the source in the tuple of sources whose spectrum will be used to "
+        "normalize the frequency-dependent data. If ``None``, the raw field data is returned "
+        "unnormalized.",
+    )
+
     courant: float = pydantic.Field(
         0.9,
         title="Courant Factor",
@@ -197,29 +205,6 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         description="String specifying the front end version number.",
     )
 
-    """ Static simulation tools."""
-
-    _hash: int = pydantic.PrivateAttr(None)
-
-    def __hash__(self) -> int:
-        """Hash a :class:`Tidy3dBaseModel` objects using its json string."""
-        if self._hash is not None:
-            return self._hash
-
-        return super().__hash__()
-
-    def _freeze(self) -> int:
-        """Computes the hash, sets ``self._hash``, and returns it."""
-        frozen_hash = hash(self)
-        self._hash = frozen_hash
-        return frozen_hash
-
-    def _unfreeze(self) -> int:
-        """Sets ``self._hash`` to ``None``, returns final hash."""
-        self._hash = None
-        final_hash = hash(self)
-        return final_hash
-
     """ Validating setup """
 
     @pydantic.validator("grid_size", always=True)
@@ -255,6 +240,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
     _unique_structure_names = assert_unique_names("structures")
     _unique_source_names = assert_unique_names("sources")
     _unique_monitor_names = assert_unique_names("monitors")
+
     # _unique_medium_names = assert_unique_names("structures", check_mediums=True)
 
     # _few_enough_mediums = validate_num_mediums()
@@ -332,6 +318,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         sources = values.get("sources")
 
         if (not structures) or (not sources):
+
             return val
 
         def warn(istruct, side):
@@ -383,7 +370,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
             return val
 
         structures = values.get("structures")
-        structures = [] if not structures else structures
+        structures = structures or []
         medium_bg = values.get("medium")
         mediums = [medium_bg] + [structure.medium for structure in structures]
 
@@ -407,11 +394,12 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
                         medium_str = f"The medium associated with structures[{medium_index-1}]"
 
                     log.warning(
-                        medium_str + f"has a frequency range: ({fmin_med:2e}, {fmax_med:2e}) (Hz)"
-                        "that does not fully cover the frequencies contained in "
+                        f"{medium_str}has a frequency range: ({fmin_med:2e}, {fmax_med:2e}) "
+                        "(Hz)that does not fully cover the frequencies contained in "
                         f"monitors[{monitor_index}]. "
                         "This can cause innacuracies in the recorded results."
                     )
+
         return val
 
     @pydantic.validator("monitors", always=True)
@@ -422,8 +410,14 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
             return val
 
         # Get simulation frequency range
+        if "sources" not in values:
+            raise ValidationError(
+                "could not validate `_warn_monitor_simulation_frequency_range` "
+                "as `sources` failed validation"
+            )
+
         source_ranges = [source.source_time.frequency_range() for source in values["sources"]]
-        if len(source_ranges) == 0:
+        if not source_ranges:
             log.warning("No sources in simulation.")
             return val
 
@@ -451,7 +445,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
             return val
 
         structures = values.get("structures")
-        structures = [] if not structures else structures
+        structures = structures or []
         medium_bg = values.get("medium")
         mediums = [medium_bg] + [structure.medium for structure in structures]
 
@@ -469,20 +463,20 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
                 lambda_min = C_0 / freq0 / n_material
 
                 for key, grid_spec in zip("xyz", (val.grid_x, val.grid_y, val.grid_z)):
-                    if isinstance(grid_spec, UniformGrid):
-
-                        if grid_spec.dl > lambda_min / MIN_GRIDS_PER_WVL:
-
-                            log.warning(
-                                f"The grid step in {key} has a value of {grid_spec.dl:.4f} (um)"
-                                ", which was detected as being large when compared to the "
-                                f"central wavelength of sources[{source_index}] "
-                                f"within the simulation medium "
-                                f"associated with structures[{medium_index + 1}], given by "
-                                f"{lambda_min:.4f} (um). "
-                                "To avoid inaccuracies, it is reccomended the grid size is reduced."
-                            )
-                    # TODO: warn about custom grid spec
+                    if (
+                        isinstance(grid_spec, UniformGrid)
+                        and grid_spec.dl > lambda_min / MIN_GRIDS_PER_WVL
+                    ):
+                        log.warning(
+                            f"The grid step in {key} has a value of {grid_spec.dl:.4f} (um)"
+                            ", which was detected as being large when compared to the "
+                            f"central wavelength of sources[{source_index}] "
+                            f"within the simulation medium "
+                            f"associated with structures[{medium_index + 1}], given by "
+                            f"{lambda_min:.4f} (um). "
+                            "To avoid inaccuracies, it is reccomended the grid size is reduced."
+                        )
+                        # TODO: warn about custom grid spec
 
         return val
 
@@ -502,9 +496,8 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
             medium=values.get("medium"),
         )
 
-        structures = values.get("structures")
-        structures = [] if not structures else structures
-        total_structures = [structure_bg] + structures
+        structures = values.get("structures") or []
+        total_structures = [structure_bg] + list(structures)
 
         # for each plane wave in the sources list
         for source in val:
@@ -520,6 +513,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
                 # make sure there is no more than one medium in the returned list
                 mediums = {medium for medium, _ in structures_merged}
                 if len(mediums) > 1:
+
                     raise SetupError(
                         f"{len(mediums)} different mediums detected on plane "
                         f"intersecting a {source.type} source. Plane must be homogeneous."
@@ -527,6 +521,22 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
 
         return val
 
+    @pydantic.validator("normalize_index", always=True)
+    def _check_normalize_index(cls, val, values):
+        """Check validity of normalize index in context of simulation.sources."""
+
+        # not normalizing
+        if val is None:
+            return val
+
+        assert val >= 0, "normalize_index can't be negative."
+        num_sources = len(values.get("sources"))
+        if num_sources > 0:
+            # No check if no sources, but it should be irrelevant anyway
+            assert val < num_sources, f"{num_sources} sources smaller than normalize_index of {val}"
+
+        return val
+
     """ Pre submit validation (before web.upload()) """
 
     def validate_pre_upload(self) -> None:
@@ -586,7 +596,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
     def _validate_run_time(self) -> None:
         """Ensures that the simulation run time is > 0."""
 
-        if not self.run_time > 0:
+        if self.run_time <= 0:
             raise SetupError(
                 "The `Simulation.run_time` parameter was left at its default value of 0.0. "
                 "For running a simulation on our servers it must be set to > 0.0."
@@ -594,8 +604,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
 
     """ Accounting """
 
-    @property
-    @cache
+    @cached_property
     def mediums(self) -> Set[MediumType]:
         """Returns set of distinct :class:`AbstractMedium` in simulation.
 
@@ -608,8 +617,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         medium_dict.update({structure.medium: None for structure in self.structures})
         return list(medium_dict.keys())
 
-    @property
-    @cache
+    @cached_property
     def medium_map(self) -> Dict[MediumType, pydantic.NonNegativeInt]:
         """Returns dict mapping medium to index in material.
         ``medium_map[medium]`` returns unique global index of :class:`AbstractMedium` in simulation.
@@ -629,7 +637,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
                 return monitor
         raise Tidy3dKeyError(f"No monitor named '{name}'")
 
-    @property
+    @cached_property
     def background_structure(self) -> Structure:
         """Returns structure representing the background of the :class:`Simulation`."""
         geometry = Box(size=(inf, inf, inf))
@@ -639,7 +647,6 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
 
     @equal_aspect
     @add_ax_if_none
-    @make_static
     def plot(
         self,
         x: float = None,
@@ -684,7 +691,6 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
 
     @equal_aspect
     @add_ax_if_none
-    @make_static
     def plot_eps(  # pylint:disable=too-many-arguments
         self,
         x: float = None,
@@ -786,22 +792,20 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
     def _get_structure_plot_params(self, mat_index: int, medium: Medium) -> PlotParams:
         """Constructs the plot parameters for a given medium in simulation.plot()."""
 
-        plot_params = plot_params_structure.copy(deep=True)
-        plot_params.linewidth = 0
+        plot_params = plot_params_structure.copy(update={"linewidth": 0})
 
         if mat_index == 0 or medium == self.medium:
             # background medium
-            plot_params.facecolor = "white"
-            plot_params.edgecolor = "white"
+            plot_params = plot_params.copy(update={"facecolor": "white", "edgecolor": "white"})
         elif isinstance(medium, PECMedium):
             # perfect electrical conductor
-            plot_params.facecolor = "gold"
-            plot_params.edgecolor = "k"
-            plot_params.linewidth = 1
+            plot_params = plot_params.copy(
+                update={"facecolor": "gold", "edgecolor": "k", "linewidth": 1}
+            )
         else:
             # regular medium
             facecolor = MEDIUM_CMAP[(mat_index - 1) % len(MEDIUM_CMAP)]
-            plot_params.facecolor = facecolor
+            plot_params = plot_params.copy(update={"facecolor": facecolor})
 
         return plot_params
 
@@ -870,7 +874,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         if alpha < 1:
             medium_shapes = self._filter_structures_plane(structures=structures, x=x, y=y, z=z)
         else:
-            structures = [self.background_structure] + structures
+            structures = [self.background_structure] + list(structures)
             medium_shapes = self._get_structures_plane(structures=structures, x=x, y=y, z=z)
 
         eps_min, eps_max = self.eps_bounds(freq=freq)
@@ -902,7 +906,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
     def eps_bounds(self, freq: float = None) -> Tuple[float, float]:
         """Compute range of (real) permittivity present in the simulation at frequency "freq"."""
 
-        medium_list = [self.medium] + self.mediums
+        medium_list = [self.medium] + list(self.mediums)
         medium_list = [medium for medium in medium_list if not isinstance(medium, PECMedium)]
         eps_list = [medium.eps_model(freq).real for medium in medium_list]
         eps_min = min(1, min(eps_list))
@@ -920,20 +924,15 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
     ) -> PlotParams:
         """Constructs the plot parameters for a given medium in simulation.plot_eps()."""
 
-        plot_params = plot_params_structure.copy(deep=True)
-        plot_params.linewidth = 0
+        plot_params = plot_params_structure.copy(update={"linewidth": 0})
         if alpha is not None:
-            plot_params.alpha = alpha
+            plot_params = plot_params.copy(update={"alpha": alpha})
 
-        # if medium == self.medium:
-        #     # background medium
-        #     plot_params.facecolor = "white"
-        #     plot_params.edgecolor = "white"
         if isinstance(medium, PECMedium):
             # perfect electrical conductor
-            plot_params.facecolor = "gold"
-            plot_params.edgecolor = "k"
-            plot_params.linewidth = 1
+            plot_params = plot_params.copy(
+                update={"facecolor": "gold", "edgecolor": "k", "linewidth": 1}
+            )
         else:
             # regular medium
             eps_medium = medium.eps_model(frequency=freq).real
@@ -941,7 +940,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
             delta_eps_max = eps_max - eps_min + 1e-5
             eps_fraction = delta_eps / delta_eps_max
             color = eps_fraction if reverse else 1 - eps_fraction
-            plot_params.facecolor = str(color)
+            plot_params = plot_params.copy(update={"facecolor": str(color)})
 
         return plot_params
 
@@ -1025,7 +1024,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         ax = self._set_plot_bounds(ax=ax, x=x, y=y, z=z)
         return ax
 
-    @property
+    @cached_property
     def num_pml_layers(self) -> List[Tuple[float, float]]:
         """Number of absorbing layers in all three axes and directions (-, +).
 
@@ -1043,7 +1042,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
 
         return num_layers
 
-    @property
+    @cached_property
     def pml_thicknesses(self) -> List[Tuple[float, float]]:
         """Thicknesses (um) of absorbers in all three axes and directions (-, +)
 
@@ -1060,7 +1059,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
             pml_thicknesses.append((thick_l, thick_r))
         return pml_thicknesses
 
-    @property
+    @cached_property
     def bounds_pml(self) -> Tuple[Tuple[float, float, float], Tuple[float, float, float]]:
         """Simulation bounds including the PML regions."""
         pml_thick = self.pml_thicknesses
@@ -1166,19 +1165,16 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
     def _make_symmetry_plot_params(self, sym_value: Symmetry) -> PlotParams:
         """Make PlotParams for symmetry."""
 
-        plot_params = plot_params_symmetry.copy(deep=True)
+        plot_params = plot_params_symmetry.copy()
 
         if sym_value == 1:
-            plot_params.facecolor = "lightsteelblue"
-            plot_params.edgecolor = "lightsteelblue"
-            plot_params.hatch = "++"
+            plot_params = plot_params.copy(
+                update={"facecolor": "lightsteelblue", "edgecolor": "lightsteelblue", "hatch": "++"}
+            )
         elif sym_value == -1:
-            # plot_params.facecolor = "rosybrown"
-            # plot_params.edgecolor = "rosybrown"
-            # plot_params.hatch = "--"
-            plot_params.facecolor = "goldenrod"
-            plot_params.edgecolor = "goldenrod"
-            plot_params.hatch = "--"
+            plot_params = plot_params.copy(
+                update={"facecolor": "goldenrod", "edgecolor": "goldenrod", "hatch": "--"}
+            )
 
         return plot_params
 
@@ -1308,8 +1304,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
             elif isinstance(boundary_edge, BlochBoundary):
                 plot_params = plot_params_bloch.copy(deep=True)
             else:
-                plot_params = PlotParams()
-                plot_params.alpha = 0
+                plot_params = PlotParams(alpha=0)
 
             # expand axis limit so that the axis ticks and labels aren't covered
             new_lim = lim
@@ -1523,7 +1518,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         # filter out any remaining None or empty shapes (shapes with area completely removed)
         return [(medium, shape) for (medium, shape, _) in background_shapes if shape]
 
-    @property
+    @cached_property
     def frequency_range(self) -> FreqBound:
         """Range of frequencies spanning all sources' frequency dependence.
 
@@ -1540,7 +1535,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
 
     """ Discretization """
 
-    @property
+    @cached_property
     def dt(self) -> float:
         """Simulation time step (distance).
 
@@ -1554,7 +1549,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         dl_avg = 1 / np.sqrt(dl_sum_inv_sq)
         return self.courant * dl_avg / C_0
 
-    @property
+    @cached_property
     def tmesh(self) -> Coords1D:
         """FDTD time stepping points.
 
@@ -1566,14 +1561,13 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         dt = self.dt
         return np.arange(0.0, self.run_time + dt, dt)
 
-    @property
+    @cached_property
     def num_time_steps(self) -> int:
         """Number of time steps in simulation."""
 
         return len(self.tmesh)
 
-    @property
-    @cache
+    @cached_property
     def grid(self) -> Grid:
         """FDTD grid spatial locations and information.
 
@@ -1594,7 +1588,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
             num_pml_layers=self.num_pml_layers,
         )
 
-    @property
+    @cached_property
     def num_cells(self) -> int:
         """Number of cells in the simulation.
 
@@ -1606,7 +1600,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
 
         return np.prod(self.grid.num_cells, dtype=np.int64)
 
-    @property
+    @cached_property
     def wvl_mat_min(self) -> float:
         """Minimum wavelength in the material.
 
@@ -1621,6 +1615,38 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         n_max, _ = AbstractMedium.eps_complex_to_nk(eps_max)
         return wvl_min / n_max
 
+    @cached_property
+    def complex_fields(self) -> bool:
+        """Whether complex fields are used in the simulation. Currently this only happens when there
+        are Bloch boundaries.
+
+        Returns
+        -------
+        bool
+            Whether the time-stepping fields are real or complex.
+        """
+        if any(isinstance(boundary[0], BlochBoundary) for boundary in self.boundary_spec.to_list):
+            return True
+        return False
+
+    @cached_property
+    def nyquist_step(self) -> int:
+        """Maximum number of discrete time steps to keep sampling below Nyquist limit.
+
+        Returns
+        -------
+        int
+            The largest ``N`` such that ``N * self.dt`` is below the Nyquist limit.
+        """
+        freq_range = self.frequency_range
+        if freq_range[1] > 0:
+            nyquist_step = int(1 / (2 * freq_range[1]) / self.dt) - 1
+            nyquist_step = max(1, nyquist_step)
+        else:
+            nyquist_step = 1
+
+        return nyquist_step
+
     def min_sym_box(self, box: Box) -> Box:  # pylint:disable=too-many-locals
         """Compute the smallest Box restricted to the first quadrant in the presence of symmetries
         that fully covers the original Box when symmetries are applied.
@@ -1645,14 +1671,13 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         for (center, sym, bmin, bmax, sim_bmin, sim_bmax) in zipped:
             if sym == 0 or center < bmin:
                 bmin_tmp, bmax_tmp = bmin, bmax
+            elif bmax < center:
+                bmin_tmp = 2 * center - bmax
+                bmax_tmp = 2 * center - bmin
             else:
-                if bmax < center:
-                    bmin_tmp = 2 * center - bmax
-                    bmax_tmp = 2 * center - bmin
-                else:
-                    # bmin <= center <= bmax
-                    bmin_tmp = center
-                    bmax_tmp = max(bmax, 2 * center - bmin)
+                # bmin <= center <= bmax
+                bmin_tmp = center
+                bmax_tmp = max(bmax, 2 * center - bmin)
             # Extend well past the simulation domain if needed, but truncate if original box
             # is too large, specifically to avoid issues with inf.
             sim_size = sim_bmax - sim_bmin
@@ -1661,13 +1686,20 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
 
         return Box.from_bounds(bmin_new, bmax_new)
 
-    def discretize(self, box: Box) -> Grid:
+    def discretize(self, box: Box, extend: bool = False, snap_zero_dim: bool = False) -> Grid:
         """Grid containing only cells that intersect with a :class:`Box`.
 
         Parameters
         ----------
         box : :class:`Box`
             Rectangular geometry within simulation to discretize.
+        extend : bool
+            If ``True``, extra pixels are added to the discretized grid if needed, such that Yee
+            grid fields can be interpolated to any point inside the ``box``.
+        snap_zero_dim : bool
+            If ``True``, and the ``box`` has size zero along a given direction, the ``grid`` is
+            defined to also have a zero-sized cell exactly centered at the ``box`` center. If
+            false, the ``simulation`` grid cell containing the ``box`` center is instead used.
 
         Returns
         -------
@@ -1678,22 +1710,26 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         if not self.intersects(box):
             log.error(f"Box {box} is outside simulation, cannot discretize")
 
-        disc_inds = self.grid.discretize_inds(box)
-        sub_cell_boundary_dict = {}
-        for axis_label, axis_inds in zip("xyz", disc_inds):
-            # copy orginal bound coords into subgrid coords
-            bound_coords = self.grid.boundaries.dict()[axis_label]
-            # axis_inds[1] + 1 because we are selecting cell boundaries not cells
-            sub_cell_boundary_dict[axis_label] = bound_coords[axis_inds[0] : axis_inds[1] + 1]
+        span_inds = self.grid.discretize_inds(box, extend=extend)
+        boundary_dict = {}
+        for idim, dim in enumerate("xyz"):
+            if snap_zero_dim and box.size[idim] == 0:
+                boundary_dict[dim] = [box.center[idim], box.center[idim]]
+            else:
+                ind_beg, ind_end = span_inds[idim]
+                # ind_end + 1 because we are selecting cell boundaries not cells
+                boundary_dict[dim] = self.grid.periodic_subspace(idim, ind_beg, ind_end + 1)
 
-        # construct sub grid
-        sub_boundaries = Coords(**sub_cell_boundary_dict)
-        return Grid(boundaries=sub_boundaries)
+        return Grid(boundaries=Coords(**boundary_dict))
 
     def epsilon(
-        self, box: Box, coord_key: str = "centers", freq: float = None
+        self,
+        box: Box,
+        coord_key: str = "centers",
+        freq: float = None,
     ) -> Dict[str, xr.DataArray]:
-        """Get array of permittivity at volume specified by box and freq
+        """Get array of permittivity at volume specified by box and freq.
+
         Parameters
         ----------
         box : :class:`Box`
@@ -1717,15 +1753,44 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         """
 
         sub_grid = self.discretize(box)
+        return self.epsilon_on_grid(grid=sub_grid, coord_key=coord_key, freq=freq)
+
+    def epsilon_on_grid(
+        self,
+        grid: Grid,
+        coord_key: str = "centers",
+        freq: float = None,
+    ) -> Dict[str, xr.DataArray]:
+        """Get array of permittivity at a given freq on a given grid.
+
+        Parameters
+        ----------
+        grid : :class:`Grid`
+            Grid specifying where to measure the permittivity.
+        coord_key : str = 'centers'
+            Specifies at what part of the grid to return the permittivity at.
+            Accepted values are ``{'centers', 'boundaries', 'Ex', 'Ey', 'Ez'}``.
+            The field values (eg. 'Ex') correspond to the correponding field locations on the yee
+            lattice. If field values are selected, the corresponding epsilon component from the
+            main diagonal of the epsilon tensor is returned. Otherwise, the average of the diagonal
+            values is returned.
+        freq : float = None
+            The frequency to evaluate the mediums at.
+            If not specified, evaluates at infinite frequency.
+        Returns
+        -------
+        xarray.DataArray
+            Datastructure containing the relative permittivity values and location coordinates.
+            For details on xarray DataArray objects,
+            refer to `xarray's Documentaton <https://tinyurl.com/2zrzsp7b>`_.
+        """
 
         def get_eps(medium: Medium, freq: float):
             """Select the correct epsilon component if field locations are requested."""
-            if coord_key[0] == "E":
-                component = ["x", "y", "z"].index(coord_key[1])
-                eps = medium.eps_diagonal(freq)[component]
-            else:
-                eps = medium.eps_model(freq)
-            return eps
+            if coord_key[0] != "E":
+                return medium.eps_model(freq)
+            component = ["x", "y", "z"].index(coord_key[1])
+            return medium.eps_diagonal(freq)[component]
 
         eps_background = get_eps(self.medium, freq)
 
@@ -1743,10 +1808,11 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
                 eps_structure = get_eps(structure.medium, freq)
                 is_inside = structure.geometry.inside(x, y, z)
                 eps_array[np.where(is_inside)] = eps_structure
-            return xr.DataArray(eps_array, coords={"x": xs, "y": ys, "z": zs}, dims=("x", "y", "z"))
+            coords = {"x": np.array(xs), "y": np.array(ys), "z": np.array(zs)}
+            return xr.DataArray(eps_array, coords=coords, dims=("x", "y", "z"))
 
         # combine all data into dictionary
-        coords = sub_grid[coord_key]
+        coords = grid[coord_key]
         return make_eps_data(coords)
 
     @classmethod
