@@ -8,8 +8,8 @@ from typing_extensions import Literal
 import pydantic
 import numpy as np
 
-from .base import Tidy3dBaseModel
-from .types import Direction, Polarization, Ax, FreqBound, Array, Axis, ArrayLike, Bound
+from .base import Tidy3dBaseModel, cached_property
+from .types import Direction, Polarization, Ax, FreqBound, ArrayLike, Axis, Bound
 from .validators import assert_plane, validate_name_str
 from .geometry import Box
 from .mode import ModeSpec
@@ -50,7 +50,11 @@ class SourceTime(ABC, Tidy3dBaseModel):
         """
 
     def spectrum(
-        self, times: Array[float], freqs: Array[float], dt: float, complex_fields: bool = False
+        self,
+        times: ArrayLike[float, 1],
+        freqs: ArrayLike[float, 1],
+        dt: float,
+        complex_fields: bool = False,
     ) -> complex:
         """Complex-valued source spectrum as a function of frequency
 
@@ -73,10 +77,12 @@ class SourceTime(ABC, Tidy3dBaseModel):
             Complex-valued array (of len(freqs)) containing spectrum at those frequencies.
         """
 
-        if complex_fields:
-            time_amps = self.amp_time(times)
-        else:
-            time_amps = np.real(self.amp_time(times))
+        times = np.array(times)
+        freqs = np.array(freqs)
+        time_amps = self.amp_time(times)
+
+        if not complex_fields:
+            time_amps = np.real(time_amps)
 
         # Cut to only relevant times
         count_times = np.where(np.abs(time_amps) / np.amax(np.abs(time_amps)) > DFT_CUTOFF)
@@ -88,7 +94,7 @@ class SourceTime(ABC, Tidy3dBaseModel):
         return dt * dft_matrix @ time_amps
 
     @add_ax_if_none
-    def plot(self, times: Array[float], ax: Ax = None) -> Ax:
+    def plot(self, times: ArrayLike[float, 1], ax: Ax = None) -> Ax:
         """Plot the complex-valued amplitude of the source time-dependence.
 
         Parameters
@@ -119,7 +125,11 @@ class SourceTime(ABC, Tidy3dBaseModel):
 
     @add_ax_if_none
     def plot_spectrum(
-        self, times: Array[float], num_freqs: int = 101, ax: Ax = None, complex_fields: bool = False
+        self,
+        times: ArrayLike[float, 1],
+        num_freqs: int = 101,
+        ax: Ax = None,
+        complex_fields: bool = False,
     ) -> Ax:
         """Plot the complex-valued amplitude of the source time-dependence.
 
@@ -185,7 +195,8 @@ class Pulse(SourceTime, ABC):
     offset: float = pydantic.Field(
         5.0,
         title="Offset",
-        description="Time delay of the maximum value of the pulse in units of 1 / ``fwidth``.",
+        description="Time delay of the maximum value of the "
+        "pulse in units of 1 / (``2pi * fwidth``).",
         ge=2.5,
     )
 
@@ -271,25 +282,25 @@ class Source(Box, ABC):
 
     name: str = pydantic.Field(None, title="Name", description="Optional name for the source.")
 
-    @property
+    @cached_property
     def plot_params(self) -> PlotParams:
         """Default parameters for plotting a Source object."""
         return plot_params_source
 
     _name_validator = validate_name_str()
 
-    @property
+    @cached_property
     def geometry(self) -> Box:
         """:class:`Box` representation of source."""
 
         return Box(center=self.center, size=self.size)
 
-    @property
+    @cached_property
     def _dir_vector(self) -> Tuple[float, float, float]:
         """Returns a vector indicating the source direction for arrow plotting, if not None."""
         return None
 
-    @property
+    @cached_property
     def _pol_vector(self) -> Tuple[float, float, float]:
         """Returns a vector indicating the source polarization for arrow plotting, if not None."""
         return None
@@ -354,7 +365,7 @@ class CurrentSource(Source, ABC):
         description="Specifies the direction and type of current component.",
     )
 
-    @property
+    @cached_property
     def _pol_vector(self) -> Tuple[float, float, float]:
         """Returns a vector indicating the source polarization for arrow plotting, if not None."""
         component = self.polarization[-1]  # 'x' 'y' or 'z'
@@ -398,7 +409,7 @@ class FieldSource(Source, ABC):
 class CustomSource(Source, ABC):
     """Implements custom current components specified by data."""
 
-    data: ArrayLike
+    data: ArrayLike[float, 2]
 
 
 class CustomFieldSource(FieldSource, CustomSource):
@@ -413,7 +424,7 @@ class PlanarSource(Source, ABC):
 
     _plane_validator = assert_plane()
 
-    @property
+    @cached_property
     def injection_axis(self):
         """Injection axis of the source."""
         return self.size.index(0.0)
@@ -444,7 +455,7 @@ class DirectionalSource(FieldSource, ABC):
         "axis.",
     )
 
-    @property
+    @cached_property
     def _dir_vector(self) -> Tuple[float, float, float]:
         """Returns a vector indicating the source direction for arrow plotting, if not None."""
         if hasattr(self, "injection_axis"):
@@ -504,7 +515,7 @@ class AngledFieldSource(DirectionalSource, ABC):
         return val
 
     # pylint: disable=no-member
-    @property
+    @cached_property
     def _dir_vector(self) -> Tuple[float, float, float]:
         """Source direction normal vector in cartesian coordinates."""
         radius = 1.0 if self.direction == "+" else -1.0
@@ -513,17 +524,19 @@ class AngledFieldSource(DirectionalSource, ABC):
         dz = radius * np.cos(self.angle_theta)
         return self.unpop_axis(dz, (dx, dy), axis=self.injection_axis)
 
-    @property
+    @cached_property
     def _pol_vector(self) -> Tuple[float, float, float]:
         """Source polarization normal vector in cartesian coordinates."""
         normal_dir = [0.0, 0.0, 0.0]
         normal_dir[self.injection_axis] = 1.0
         propagation_dir = list(self._dir_vector)
-        pol_vector_s = np.cross(normal_dir, propagation_dir)
-        if np.all(pol_vector_s == 0.0):
-            pol_vector_s = np.array((0, 1, 0)) if self.injection_axis == 0 else np.array((1, 0, 0))
-        pol_vector_p = np.cross(propagation_dir, pol_vector_s)
-        pol_vector_p = np.array(pol_vector_p) / np.linalg.norm(pol_vector_p)
+        if self.angle_theta == 0.0:
+            pol_vector_p = np.array((0, 1, 0)) if self.injection_axis == 0 else np.array((1, 0, 0))
+            pol_vector_p = self.rotate_points(pol_vector_p, normal_dir, angle=self.angle_phi)
+        else:
+            pol_vector_s = np.cross(normal_dir, propagation_dir)
+            pol_vector_p = np.cross(propagation_dir, pol_vector_s)
+            pol_vector_p = np.array(pol_vector_p) / np.linalg.norm(pol_vector_p)
         return self.rotate_points(pol_vector_p, propagation_dir, angle=self.pol_angle)
 
 
@@ -557,17 +570,17 @@ class ModeSource(DirectionalSource, PlanarSource):
         "``num_modes`` in the solver will be set to ``mode_index + 1``.",
     )
 
-    @property
+    @cached_property
     def angle_theta(self):
         """Polar angle of propagation."""
         return self.mode_spec.angle_theta
 
-    @property
+    @cached_property
     def angle_phi(self):
         """Azimuth angle of propagation."""
         return self.mode_spec.angle_phi
 
-    @property
+    @cached_property
     def _dir_vector(self) -> Tuple[float, float, float]:
         """Source direction normal vector in cartesian coordinates."""
         radius = 1.0 if self.direction == "+" else -1.0
@@ -589,16 +602,6 @@ class PlaneWave(AngledFieldSource, PlanarSource):
     >>> pw_source = PlaneWave(size=(inf,0,inf), source_time=pulse, pol_angle=0.1, direction='+')
     """
 
-    # @pydantic.validator("angle_theta", always=True)
-    # def normal_incidence(cls, val):
-    #     """Raise not implemented error if not normal incidence."""
-    #     if val > 0.0:
-    #         raise NotImplementedError(
-    #             "Plane wave with off-normal incidence requires Bloch "
-    #             "boundary conditions, which are not yet implemented."
-    #         )
-    #     return val
-
 
 class GaussianBeam(AngledFieldSource, PlanarSource):
     """Guassian distribution on finite extent plane.
@@ -670,5 +673,10 @@ class TFSF(AngledFieldSource, VolumeSource):
 
 # sources allowed in Simulation.sources
 SourceType = Union[
-    UniformCurrentSource, PointDipole, GaussianBeam, AstigmaticGaussianBeam, ModeSource, PlaneWave
+    UniformCurrentSource,
+    PointDipole,
+    GaussianBeam,
+    AstigmaticGaussianBeam,
+    ModeSource,
+    PlaneWave,
 ]
