@@ -1,6 +1,6 @@
 """Fit PoleResidue Dispersion models to optical NK data based on web service
 """
-from typing import Tuple, List
+from typing import Tuple
 from enum import Enum
 import requests
 from pydantic import PositiveInt, NonNegativeFloat, PositiveFloat, Field
@@ -37,7 +37,7 @@ class AdvancedFitterParam(Tidy3dBaseModel):
         units=HERTZ,
     )
     bound_eps_inf: float = Field(
-        1.0,
+        10.0,
         title="Upper bound of epsilon at infinity frequency",
         description="Upper bound of epsilon at infinity frequency. It must be no less than 1.",
         ge=1,
@@ -60,25 +60,24 @@ class AdvancedFitterParam(Tidy3dBaseModel):
 
 # FitterData will be used internally
 class FitterData(AdvancedFitterParam):
-    """Data class for request body of Fitter where dipsersion data is input through list."""
+    """Data class for request body of Fitter where dipsersion data is input through tuple."""
 
-    wvl_um: List[float] = Field(
+    wvl_um: Tuple[float, ...] = Field(
         ...,
         title="Wavelengths",
-        description="A list of wavelengths for dispersion data.",
+        description="A set of wavelengths for dispersion data.",
         units=MICROMETER,
     )
-    n_data: List[float] = Field(
+    n_data: Tuple[float, ...] = Field(
         ...,
         title="Index of refraction",
-        description="Real part of the complex index of refraction.",
+        description="Real part of the complex index of refraction at each wavelength.",
     )
-    k_data: List[float] = Field(
-        ...,
+    k_data: Tuple[float, ...] = Field(
+        None,
         title="Extinction coefficient",
-        description="Imaginary part of the complex index of refraction.",
+        description="Imaginary part of the complex index of refraction at each wavelength.",
     )
-
     num_poles: PositiveInt = Field(
         1, title="Number of poles", description="Number of poles in model."
     )
@@ -135,14 +134,10 @@ class StableDispersionFitter(DispersionFitter):
         """
 
         _env = config_env
-        if config_env == "default":
+        if _env == "default":
             from ...web.config import DEFAULT_CONFIG  # pylint:disable=import-outside-toplevel
 
-            if "dev" in DEFAULT_CONFIG.web_api_endpoint:
-                _env = "dev"
-            else:
-                _env = "prod"
-
+            _env = "dev" if "dev" in DEFAULT_CONFIG.web_api_endpoint else "prod"
         return URL_ENV[_env]
 
     @staticmethod
@@ -159,14 +154,14 @@ class StableDispersionFitter(DispersionFitter):
         headers = {"Authorization": access_token["Authorization"]}
 
         # test connection
-        resp = requests.get(url_server + "/health")
+        resp = requests.get(f"{url_server}/health")
         try:
             resp.raise_for_status()
         except Exception as e:
             raise WebError("Connection to the server failed. Please try again.") from e
 
         # test authorization
-        resp = requests.get(url_server + "/health/access", headers=headers)
+        resp = requests.get(f"{url_server}/health/access", headers=headers)
         try:
             resp.raise_for_status()
         except Exception as e:
@@ -198,23 +193,30 @@ class StableDispersionFitter(DispersionFitter):
         -------
         :class:`FitterData`
             Data class for request body of Fitter where dipsersion
-            data is input through list.
+            data is input through tuple.
         """
 
         # set up bound_f, bound_amp
         if advanced_param.bound_f is None:
-            advanced_param.bound_f = self.frequency_range[1] * BOUND_MAX_FACTOR
+            new_bound_f = self.frequency_range[1] * BOUND_MAX_FACTOR
+            advanced_param = advanced_param.copy(update={"bound_f": new_bound_f})
         if advanced_param.bound_amp is None:
-            advanced_param.bound_amp = self.frequency_range[1] * BOUND_MAX_FACTOR
+            new_bound_amp = self.frequency_range[1] * BOUND_MAX_FACTOR
+            advanced_param = advanced_param.copy(update={"bound_amp": new_bound_amp})
 
         wvl_um, n_data, k_data = self._filter_wvl_range(
             wvl_min=self.wvl_range[0], wvl_max=self.wvl_range[1]
         )
 
+        if self.lossy:
+            k_data = k_data.tolist()
+        else:
+            k_data = None
+
         web_data = FitterData(
             wvl_um=wvl_um.tolist(),
             n_data=n_data.tolist(),
-            k_data=k_data.tolist(),
+            k_data=k_data,
             num_poles=num_poles,
             num_tries=num_tries,
             tolerance_rms=tolerance_rms,
@@ -260,11 +262,7 @@ class StableDispersionFitter(DispersionFitter):
         # setup web_data
         web_data = self._setup_webdata(num_poles, num_tries, tolerance_rms, advanced_param)
 
-        resp = requests.post(
-            url_server + "/dispersion/fit",
-            headers=headers,
-            data=web_data.json(),
-        )
+        resp = requests.post(f"{url_server}/dispersion/fit", headers=headers, data=web_data.json())
 
         try:
             resp.raise_for_status()
